﻿using Kairai;

using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Numerics;
using System.Runtime.InteropServices.WindowsRuntime;
using System.Threading;
using System.Threading.Tasks;
using Windows.ApplicationModel;
using Windows.Foundation;
using Windows.Foundation.Collections;
using Windows.Graphics.Display;
using Windows.Storage;
using Windows.Storage.Streams;
using Windows.Storage.Pickers;
using Windows.System.Threading;
using Windows.UI.Core;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;
using Windows.UI.Xaml.Controls.Primitives;
using Windows.UI.Xaml.Data;
using Windows.UI.Xaml.Input;
using Windows.UI.Xaml.Media;
using Windows.UI.Xaml.Navigation;

// 空白ページのアイテム テンプレートについては、http://go.microsoft.com/fwlink/?LinkId=234238 を参照してください

namespace KumaApp
{
    /// <summary>
    /// それ自体で使用できる空白ページまたはフレーム内に移動できる空白ページ。
    /// </summary>
    public sealed partial class MainPage : Page
    {
        bool  m_windowVisible;
        Util m_util;
        KairaiApi m_kairai;
        int m_count;
        GameStatus m_textStatus;
        		
	    void KairaiCallback(KairaiApi kairai)
        {
            m_util.Callback();
        }

        void ShowText(GameStatus status)
        {
            if(status == m_textStatus)
            {
                return;
            }
            m_textStatus = status;

            var show = Visibility.Visible;
            var hide = Visibility.Collapsed;
            var start = hide;
            var gameOver = hide;
            var restart = hide;
            var goal = hide;

            switch (m_util.Status)
            {
                case GameStatus.Wait:
                    start = show;
                    break;

                case GameStatus.GameOver:
                    gameOver = show;
                    break;

                case GameStatus.Restart:
                    restart = show;
                    break;

                case GameStatus.Goal:
                    goal = show;
                    break;
            }
            if (TextStart.Visibility != start)
            {
                TextStart.Visibility = start;
            }
            if (TextGameOver.Visibility != gameOver)
            {
                TextGameOver.Visibility = gameOver;
            }
            if (TextRestart.Visibility != restart)
            {
                TextRestart.Visibility = restart;
            }
            if (TextGoal.Visibility != goal)
            {
                TextGoal.Visibility = goal;
            }
        }

        public void TimerTick(object sender, object e)
        {
            switch (m_kairai.GetKairaiState())
            {
                case KAIRAI_STATE.KAIRAI_INIT:
                    if (m_kairai.IsBusy())
                    {
                        break;
                    }
                    m_count = 0;
                    m_util.LoadMeshes();
                    break;

                case KAIRAI_STATE.KAIRAI_READY:
                    switch (m_util.Status)
                    {
                        case GameStatus.Wait:
                            ShowText(m_util.Status);
                            m_count++;
                            if (m_count >= 3)
                            {
                                m_util.Status = GameStatus.Ready;
                            }
                            break;

                        case GameStatus.GameOver:
                        case GameStatus.Restart:
                        case GameStatus.Ready:
                        case GameStatus.Goal:
                            ShowText(m_util.Status);
                            break;
                    }
                    break;
            }
        }

        public MainPage()
        {
            this.InitializeComponent();

            m_kairai = new KairaiApi();
            m_util = new Util(m_kairai);
            m_textStatus = GameStatus.Max;
 
        	DisplayInformation displayInf = DisplayInformation.GetForCurrentView();
	        displayInf.DpiChanged += OnDpiChanged;
            displayInf.OrientationChanged += OnOrientationChanged;
        	DisplayInformation.DisplayContentsInvalidated += OnDisplayContentsInvalidated;

        	swapChainPanel.CompositionScaleChanged += OnCompositionScaleChanged;
            swapChainPanel.SizeChanged += OnSwapChainPanelSizeChanged;

            CoreWindow window = Window.Current.CoreWindow;

            window.KeyDown += OnKeyDown;
            window.PointerPressed += OnPointerPressed;
            window.PointerMoved += OnPointerMoved;
            window.PointerReleased += OnPointerReleased;
            window.PointerWheelChanged += OnPointerWheelChanged;
            window.VisibilityChanged += OnVisibilityChanged;

            m_kairai.Initialize(swapChainPanel);
            m_kairai.SetCallback(KairaiCallback);

            //Timer
	        DispatcherTimer timer = new DispatcherTimer();
            timer.Tick += TimerTick;
            timer.Interval = new TimeSpan(0, 0, 1);
	        timer.Start(); 
        }

        void OnVisibilityChanged(CoreWindow sender, VisibilityChangedEventArgs args)
        {
            m_windowVisible = args.Visible;
            if (m_windowVisible)
            {
                m_kairai.StartRenderLoop();
            }
            else
            {
                m_kairai.StopRenderLoop();
            }
        }
        void OnKeyDown(CoreWindow sender, KeyEventArgs args)
        {
        }
        public void SaveInternalState()
        {
            m_kairai.SaveInternalState();
        }
        public void LoadInternalState()
        {
            m_kairai.StartRenderLoop();
        }
        void OnPointerPressed(object sender, PointerEventArgs e)
        {
            m_util.Attack(e.CurrentPoint.Position);
        }
        void OnPointerMoved(Object sender, PointerEventArgs e)
        {
            m_util.MoveKuma(e.CurrentPoint.Position);
        }
        void OnPointerReleased(Object sender, PointerEventArgs e)
        {
        }
        void OnPointerWheelChanged(Object sender, PointerEventArgs e)
        {
            m_kairai.OnPointerWheelChanged(sender, e);
        }
        void OnDpiChanged(DisplayInformation sender, Object args)
        {
            m_kairai.OnDpiChanged(sender, args);
        }
        void OnOrientationChanged(DisplayInformation sender, Object args)
        {
            m_kairai.OnOrientationChanged(sender, args);
        }
        void OnDisplayContentsInvalidated(DisplayInformation sender, Object args)
        {
            m_kairai.OnDisplayContentsInvalidated(sender, args);
        }
        void OnCompositionScaleChanged(SwapChainPanel sender, Object args)
        {
            m_kairai.OnCompositionScaleChanged(sender, args);
        }
        void OnSwapChainPanelSizeChanged(Object sender, SizeChangedEventArgs e)
        {
            m_kairai.OnSwapChainPanelSizeChanged(sender, e);
        }
    }
}
